#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Budy
# Copyright (c) 2008-2024 Hive Solutions Lda.
#
# This file is part of Hive Budy.
#
# Hive Budy is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Budy is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Budy. If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__copyright__ = "Copyright (c) 2008-2024 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

import logging
import unittest

import appier

import budy


class ReferralTest(unittest.TestCase):
    def setUp(self):
        self.app = budy.BudyApp(level=logging.ERROR)

    def tearDown(self):
        self.app.unload()
        adapter = appier.get_adapter()
        adapter.drop_db()

    def test_basic(self):
        referral = budy.Referral(name="name")
        referral.save()

        referral = budy.Referral.get(name="name")

        self.assertNotEqual(referral, None)
        self.assertEqual(referral.name, "name")
        self.assertEqual(referral.id, 1)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Budy
# Copyright (c) 2008-2024 Hive Solutions Lda.
#
# This file is part of Hive Budy.
#
# Hive Budy is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Budy is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Budy. If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__copyright__ = "Copyright (c) 2008-2024 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

import mimetypes

import appier

import budy

from . import root


class MediaAPIController(root.RootAPIController):
    @appier.route("/api/media", "GET", json=True)
    def list(self):
        object = appier.get_object(alias=True, find=True)
        media = budy.Media.find(map=True, **object)
        return media

    @appier.route("/api/media/<int:id>/data", "GET", json=True)
    def data(self, id):
        media = budy.Media.get(id=id, fields=("file",), rules=False)
        file = media.file
        if not file:
            raise appier.NotFoundError(message="File not found for media '%d'" % id)
        return self.send_file(
            file.data,
            name="%d" % id,
            content_type=file.mime,
            etag=file.etag,
            cache=True,
        )

    @appier.route("/api/media/<int:id>/data.<str:format>", "GET", json=True)
    def data_format(self, id, format):
        background = self.field("background", None)
        quality = self.field("quality", 90, cast=int)
        media = budy.Media.get(id=id, fields=("file",), rules=False)
        file = media.file
        if not file:
            raise appier.NotFoundError(message="File not found for media '%d'" % id)
        extension = mimetypes.guess_extension(file.mime or "")
        mime, _encoding = mimetypes.guess_type("data." + format)
        if extension and extension[1:] == format:
            return self.send_file(
                file.data,
                name="%d.%s" % (id, format),
                content_type=file.mime,
                etag=file.etag,
                cache=True,
            )
        buffer = media.convert_image(
            format, background=background, **dict(quality=quality)
        )
        return self.send_file(
            buffer.getvalue(),
            name="%d.%s" % (id, format),
            content_type=mime,
            etag=file.etag,
            cache=True,
        )



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Budy
# Copyright (c) 2008-2024 Hive Solutions Lda.
#
# This file is part of Hive Budy.
#
# Hive Budy is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Budy is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Budy. If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__copyright__ = "Copyright (c) 2008-2024 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

import time

import appier

import budy

from . import root


class SeeplusAPIController(root.RootAPIController):
    @appier.route("/api/seeplus/update", "POST", json=True)
    def update(self):
        key = self.field("token", None)
        key = self.field("key", key)
        key = self.request.get_header("X-Seeplus-Key", key)
        _key = appier.conf("SEEPLUS_KEY", None)
        if _key and not _key == key:
            raise appier.SecurityError(message="Mismatch in Seeplus key", code=401)
        object = appier.get_object()
        event = object.get("event", "OrderManagement.StatusChanged")
        data = object.get("data", {})
        result = "Ignored"
        if event == "OrderManagement.StatusChanged":
            self._status_change_s(data)
            result = "Handled"
        return dict(result=result)

    def _status_change_s(self, data):
        for key in ("code", "status"):
            appier.verify(
                key in data,
                message="Missing '%s' in Seeplus data payload" % key,
                code=400,
            )
        reference, status = data["code"], data["status"]
        order = budy.Order.get(reference=reference)
        seeplus_status = order.meta.get("seeplus_status", None)
        seeplus_timestamp = order.meta.get("seeplus_timestamp", None)
        seeplus_updates = order.meta.get("seeplus_updates", [])
        if not seeplus_timestamp:
            raise appier.OperationalError(message="Order not imported in Seeplus")
        if status == seeplus_status:
            raise appier.OperationalError(
                message="Order already in status '%s'" % status
            )
        status_timestamp = time.time()
        seeplus_updates.append(dict(status=status, timestamp=status_timestamp))
        order.meta.update(
            seeplus_status=status,
            seeplus_update=status_timestamp,
            seeplus_updates=seeplus_updates,
        )
        order.save()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Budy
# Copyright (c) 2008-2024 Hive Solutions Lda.
#
# This file is part of Hive Budy.
#
# Hive Budy is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Budy is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Budy. If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__copyright__ = "Copyright (c) 2008-2024 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

import appier

import budy

from . import root


class OrderAPIController(root.RootAPIController):
    @appier.route("/api/orders", "GET", json=True)
    @appier.ensure(token="admin")
    def list(self):
        object = appier.get_object(alias=True, find=True, sort=[("id", -1)])
        orders = budy.Order.find(eager_l=True, map=True, **object)
        return orders

    @appier.route("/api/orders/complex.csv", "GET")
    @appier.ensure(token="admin")
    def complex_csv(self):
        start = self.field("start", cast=int)
        end = self.field("end", cast=int)
        paid = self.field("paid", True, cast=bool)
        object = appier.get_object(alias=True, find=True, limit=0, sort=[("id", -1)])
        id = object.get("id", {})
        if start:
            id["$gte"] = start
        if end:
            id["$lte"] = end
        if start or end:
            object["id"] = id
        if paid:
            object["paid"] = True
        orders = budy.Order.find(**object)
        orders_s = [
            (
                "id",
                "reference",
                "date",
                "status",
                "email",
                "account",
                "store",
                "product",
                "gender",
                "size",
                "quantity",
                "total",
                "taxes",
                "currency",
                "first_name",
                "last_name",
                "billing_address",
                "billing_city",
                "billing_state",
                "billing_postal_code",
                "billing_country",
                "billing_phone",
                "shipping_address",
                "shipping_city",
                "shipping_state",
                "shipping_postal_code",
                "shipping_country",
                "shipping_phone",
                "shipping_cost",
            )
        ]
        for order in orders:
            for line in order.lines:
                if not line.product:
                    continue
                account = order.account
                shipping_address = order.shipping_address
                billing_address = order.billing_address
                shipping_cost = order.shipping_cost
                order_s = (
                    order.id,
                    order.reference,
                    order.created_d.strftime("%d/%m/%Y"),
                    order.status,
                    order.email,
                    account.username,
                    order.store and order.store.name,
                    line.product.short_description,
                    line.product.gender,
                    line.size,
                    line.quantity,
                    line.total,
                    line.taxes,
                    line.currency,
                    billing_address.first_name,
                    billing_address.last_name,
                    billing_address.address,
                    billing_address.city,
                    billing_address.state,
                    billing_address.postal_code,
                    billing_address.country,
                    billing_address.phone_number,
                    shipping_address and shipping_address.address,
                    shipping_address and shipping_address.city,
                    shipping_address and shipping_address.state,
                    shipping_address and shipping_address.postal_code,
                    shipping_address and shipping_address.country,
                    shipping_address and shipping_address.phone_number,
                    shipping_cost,
                )
                orders_s.append(order_s)
        result = appier.serialize_csv(orders_s, delimiter=",")
        self.content_type("text/csv")
        return result

    @appier.route("/api/orders/ctt.csv", "GET")
    @appier.ensure(token="admin")
    def ctt_csv(self):
        start = self.field("start", cast=int)
        end = self.field("end", cast=int)
        paid = self.field("paid", True, cast=bool)
        sms = self.field("sms", False, cast=bool)
        quantity = self.field("quantity", 1, cast=int)
        weight = self.field("weight", 100, cast=int)
        object = appier.get_object(alias=True, find=True, limit=0, sort=[("id", -1)])
        id = object.get("id", {})
        if start:
            id["$gte"] = start
        if end:
            id["$lte"] = end
        if start or end:
            object["id"] = id
        if paid:
            object["paid"] = True
        orders = self.admin_part._find_view(budy.Order, **object)
        orders_s = []
        for order in orders:
            shipping_address = order.shipping_address
            postal_code = shipping_address.postal_code or ""
            if not "-" in postal_code:
                postal_code += "-"
            weight_s = "%.2f" % (order.quantity * weight)
            weight_s = weight_s.replace(".", ",")
            line = dict(
                reference=order.reference,
                quantity=int(order.quantity) if quantity == None else quantity,
                weight=weight_s,
                price="0ue",
                destiny=shipping_address.full_name[:60],
                title=order.account.title[:20],
                name=shipping_address.full_name[:60],
                address=shipping_address.address[:60],
                town=shipping_address.city[:50],
                zip_code_4=postal_code.split("-", 1)[0][:4],
                zip_code_3=postal_code.split("-", 1)[1][:3],
                not_applicable_1="",
                observations="",
                back=0,
                document_code="",
                phone_number=(shipping_address.phone_number or "").replace("+", "00")[
                    :15
                ],
                saturday=0,
                email=(order.email or "")[:200],
                country=order.country,
                fragile=0,
                not_applicable_2="",
                document_collection="",
                code_email="",
                mobile_phone=(shipping_address.phone_number or "").replace("+", "00")[
                    :15
                ],
                second_delivery=0,
                delivery_date="",
                return_signed_document=0,
                expeditor_instructions=0,
                sms=1 if sms else 0,
                not_applicable_3="",
                printer="",
                ticket_machine="",
                at_code="",
            )
            order_s = (
                line["reference"],
                str(line["quantity"]),
                line["weight"],
                line["price"],
                line["destiny"],
                line["title"],
                line["name"],
                line["address"],
                line["town"],
                line["zip_code_4"],
                line["zip_code_3"],
                line["not_applicable_1"],
                line["observations"],
                str(line["back"]),
                line["document_code"],
                line["phone_number"],
                str(line["saturday"]),
                line["email"],
                line["country"],
                str(line["fragile"]),
                line["not_applicable_2"],
                line["document_collection"],
                line["code_email"],
                line["mobile_phone"],
                str(line["second_delivery"]),
                line["delivery_date"],
                str(line["return_signed_document"]),
                str(line["expeditor_instructions"]),
                str(line["sms"]),
                line["not_applicable_3"],
                line["printer"],
                line["ticket_machine"],
                line["at_code"],
            )
            orders_s.append(order_s)
        result = appier.serialize_csv(
            orders_s, encoding="Cp1252", errors="ignore", delimiter="+"
        )
        result = appier.legacy.bytes(result, encoding="Cp1252", errors="ignore")
        self.content_type("text/csv")
        return result

    @appier.route("/api/orders/<str:key>", "GET", json=True)
    def show(self, key):
        order = budy.Order.get(key=key)
        order.refresh_s(
            currency=order.payment_currency or self.currency,
            country=order.shipping_country or self.country,
        )
        order = order.reload(
            eager=("lines.product.images", "lines.product.brand"), map=True
        )
        return order

    @appier.route("/api/orders/<str:key>/store", "PUT", json=True)
    @appier.ensure(token="user")
    def set_store(self, key):
        data = appier.request_json()
        store_id = data["store_id"]
        store = budy.Store.get(id=store_id)
        order = budy.Order.get(key=key, rules=False)
        order.store = store
        order.save()
        order = order.reload(map=True)
        return order

    @appier.route("/api/orders/<str:key>/shipping_address", "PUT", json=True)
    @appier.ensure(token="user")
    def set_shipping_address(self, key):
        data = appier.request_json()
        unset_store = data.get("unset_store", True)
        address = budy.Address.new()
        address.save()
        order = budy.Order.get(key=key, rules=False)
        order.shipping_address = address
        if unset_store:
            order.store = None
        order.save()
        order = order.reload(map=True)
        return order

    @appier.route("/api/orders/<str:key>/billing_address", "PUT", json=True)
    @appier.ensure(token="user")
    def set_billing_address(self, key):
        data = appier.request_json()
        unset_store = data.get("unset_store", False)
        address = budy.Address.new()
        address.save()
        order = budy.Order.get(key=key, rules=False)
        order.billing_address = address
        if unset_store:
            order.store = None
        order.save()
        order = order.reload(map=True)
        return order

    @appier.route("/api/orders/<str:key>/store_shipping", "PUT", json=True)
    @appier.ensure(token="user")
    def set_store_shipping(self, key):
        order = budy.Order.get(key=key, rules=False)
        order.verify_store()
        store = order.store
        address = store.address.clone()
        address.save()
        order.shipping_address = address
        order.save()
        order = order.reload(map=True)
        return order

    @appier.route("/api/orders/<str:key>/store_billing", "PUT", json=True)
    @appier.ensure(token="user")
    def set_store_billing(self, key):
        order = budy.Order.get(key=key, rules=False)
        order.verify_store()
        store = order.store
        address = store.address.clone()
        address.save()
        order.billing_address = address
        order.save()
        order = order.reload(map=True)
        return order

    @appier.route("/api/orders/<str:key>/ip_address", "PUT", json=True)
    @appier.ensure(token="user")
    def set_ip_address(self, key):
        data = appier.request_json()
        ip_address = data["ip_address"]
        order = budy.Order.get(key=key, rules=False)
        order.set_ip_address_s(ip_address)
        order = order.reload(map=True)
        return order

    @appier.route("/api/orders/<str:key>/email", "PUT", json=True)
    @appier.ensure(token="user")
    def set_email(self, key):
        data = appier.request_json()
        email = data["email"]
        order = budy.Order.get(key=key, rules=False)
        order.email = email
        order.save()
        order = order.reload(map=True)
        return order

    @appier.route("/api/orders/<str:key>/gift_wrap", "PUT", json=True)
    @appier.ensure(token="user")
    def set_gift_wrap(self, key):
        data = appier.request_json()
        gift_wrap = data["gift_wrap"]
        order = budy.Order.get(key=key, rules=False)
        order.gift_wrap = gift_wrap
        order.save()
        order = order.reload(map=True)
        return order

    @appier.route("/api/orders/<str:key>/referral", "PUT", json=True)
    @appier.ensure(token="user")
    def set_referral(self, key):
        data = appier.request_json()
        referral_name = data["name"]
        order = budy.Order.get(key=key, rules=False)
        referral = budy.Referral.get(name=referral_name)
        order.set_referral_s(referral)
        order = order.reload(map=True)
        return order

    @appier.route("/api/orders/<str:key>/voucher", "PUT", json=True)
    @appier.ensure(token="user")
    def set_voucher(self, key):
        strict = self.field("strict", False, cast=bool)
        empty_vouchers = self.field("empty_vouchers", True, cast=bool)
        data = appier.request_json()
        voucher_key = data["key"]
        order = budy.Order.get(key=key, rules=False)
        if empty_vouchers:
            order.empty_vouchers_s()
        voucher = budy.Voucher.get(key=voucher_key, raise_e=strict)
        if not voucher:
            return
        order.set_voucher_s(voucher)
        order = order.reload(map=True)
        return order

    @appier.route("/api/orders/<str:key>/meta", "PUT", json=True)
    @appier.ensure(token="user")
    def set_meta(self, key):
        name = self.field("name", mandatory=True, not_empty=True)
        value = self.field("value", mandatory=True)
        order = budy.Order.get(key=key, rules=False)
        order.set_meta_s(name, value)
        order = order.reload(map=True)
        return order

    @appier.route("/api/orders/<str:key>/account", "PUT", json=True)
    @appier.ensure(token="user")
    def set_account(self, key):
        order = budy.Order.get(key=key, rules=False)
        account = budy.BudyAccount.from_session()
        order.set_account_s(account)
        order = order.reload(map=True)
        return order

    @appier.route("/api/orders/<str:key>/wait_payment", "PUT", json=True)
    @appier.ensure(token="user")
    def wait_payment(self, key):
        empty_bag = self.field("empty_bag", True, cast=bool)
        order = budy.Order.get(key=key, rules=False)
        result = order.wait_payment_s(notify=True)
        bag = budy.Bag.from_session()
        if empty_bag and bag:
            bag.empty_s()
        order = order.reload(map=True)
        redirect_url = result if appier.legacy.is_string(result) else None
        return dict(redirect_url=redirect_url, order=order)

    @appier.route("/api/orders/<str:key>/pay", "PUT", json=True)
    @appier.ensure(token="user")
    def pay(self, key):
        data = appier.request_json()
        empty_bag = self.field("empty_bag", True, cast=bool)
        order = budy.Order.get(key=key, rules=False)
        result = order.pay_s(payment_data=data, notify=True)
        bag = budy.Bag.from_session()
        if empty_bag and bag:
            bag.empty_s()
        order = order.reload(map=True)
        redirect_url = result if appier.legacy.is_string(result) else None
        return dict(redirect_url=redirect_url, order=order)

    @appier.route("/api/orders/<str:key>/end_pay", "PUT", json=True)
    @appier.ensure(token="user")
    def end_pay(self, key):
        data = appier.request_json()
        empty_bag = self.field("empty_bag", True, cast=bool)
        order = budy.Order.get(key=key, rules=False)
        result = order.end_pay_s(payment_data=data, strict=True, notify=True)
        bag = budy.Bag.from_session()
        if empty_bag and bag:
            bag.empty_s()
        order = order.reload(map=True)
        redirect_url = result if appier.legacy.is_string(result) else None
        return dict(redirect_url=redirect_url, order=order)

    @appier.route("/api/orders/<str:key>/cancel", "PUT", json=True)
    @appier.ensure(token="user")
    def cancel(self, key):
        data = appier.request_json()
        order = budy.Order.get(key=key, rules=False)
        result = order.cancel_s(data, notify=True)
        order = order.reload(map=True)
        redirect_url = result if appier.legacy.is_string(result) else None
        return dict(redirect_url=redirect_url, order=order)

    @appier.route("/api/orders/<str:key>/lines.csv", "GET")
    @appier.ensure(token="admin")
    def lines_csv(self, key):
        order = budy.Order.get(key=key)
        lines_s = [
            (
                "id",
                "product",
                "product_id",
                "quantity",
                "size",
                "scale",
                "gender",
                "line_total",
                "currency",
                "order_id",
                "order_reference",
                "order_total",
                "order_status",
                "account",
                "date",
            )
        ]
        for line in order.lines:
            if not line.product:
                continue
            line_s = (
                line.id,
                line.product.id,
                line.product.product_id,
                line.quantity,
                line.size,
                line.scale,
                line.product.gender,
                line.total,
                line.currency,
                order.id,
                order.reference,
                order.total,
                order.status,
                order.account.username,
                order.created_d.strftime("%d/%m/%Y"),
            )
            lines_s.append(line_s)

        result = appier.serialize_csv(lines_s, delimiter=",")
        self.content_type("text/csv")
        return result



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Budy
# Copyright (c) 2008-2024 Hive Solutions Lda.
#
# This file is part of Hive Budy.
#
# Hive Budy is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Budy is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Budy. If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__copyright__ = "Copyright (c) 2008-2024 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

import appier

import budy

from . import root


class AccountAPIController(root.RootAPIController):
    @appier.route("/api/accounts", "GET", json=True)
    @appier.ensure(token="admin")
    def list(self):
        object = appier.get_object(alias=True, find=True)
        accounts = budy.BudyAccount.find(map=True, **object)
        return accounts

    @appier.route("/api/accounts", "POST", json=True)
    def create(self):
        pre_enabled = self.field("pre_enabled", False, cast=bool)
        account = budy.BudyAccount.new()
        account.type = budy.BudyAccount.USER_TYPE
        account.password_confirm = account.password
        account.enabled = pre_enabled
        account.save()
        account = account.reload(rules=True, map=True)
        return account

    @appier.route("/api/accounts/confirm/<str:token>", "GET", json=True)
    def confirm(self, token):
        account = budy.BudyAccount.get(confirmation_token=token)
        account.confirm_s()
        return account

    @appier.route("/api/accounts/recover/<str:identifier>", "GET", json=True)
    def recover(self, identifier):
        budy.BudyAccount.recover(identifier)

    @appier.route("/api/accounts/reset", "POST", json=True)
    def reset(self):
        token = self.field("token", mandatory=True)
        password = self.field("password", mandatory=True)
        budy.BudyAccount.reset(token, password, password)

    @appier.route("/api/accounts/me", "GET", json=True)
    @appier.ensure(token="user")
    def me(self):
        account = budy.BudyAccount.from_session(map=True)
        return account

    @appier.route("/api/accounts/me", "PUT", json=True)
    @appier.ensure(token="user")
    def update_me(self):
        account = budy.BudyAccount.from_session()
        account.apply()
        account.save()
        account = account.map()
        return account

    @appier.route("/api/accounts/me/avatar", "GET", json=True)
    @appier.ensure(token="user")
    def avatar_me(self):
        account = budy.BudyAccount.from_session(rules=False)
        return account._send_avatar()

    @appier.route("/api/accounts/me/orders", "GET", json=True)
    @appier.ensure(token="user")
    def orders_me(self):
        object = appier.get_object(alias=True, find=True, limit=0, sort=[("id", -1)])
        account = budy.BudyAccount.from_session()
        orders = budy.Order.find(
            account=account.id, paid=True, eager=("lines",), map=True, **object
        )
        return orders

    @appier.route("/api/accounts/me/addresses", "GET", json=True)
    @appier.ensure(token="user")
    def addresses_me(self):
        account = budy.BudyAccount.from_session(eager=("addresses",), map=True)
        return account["addresses"]

    @appier.route("/api/accounts/me/addresses", "POST", json=True)
    @appier.ensure(token="user")
    def create_address_me(self):
        address = budy.Address.new()
        address.save()
        account = budy.BudyAccount.from_session(rules=False)
        account.addresses.append(address)
        account.save()
        address = address.map()
        return address

    @appier.route("/api/accounts/me/addresses/<str:key>", "DELETE", json=True)
    @appier.ensure(token="user")
    def delete_address_me(self, key):
        address = budy.Address.get(key=key)
        account = budy.BudyAccount.from_session(rules=False)
        account.addresses.remove(address.id)
        account.save()
        address.delete()

    @appier.route("/api/accounts/simple.csv", "GET")
    @appier.ensure(token="admin")
    def simple_csv(self):
        object = appier.get_object(alias=True, find=True, limit=0)
        accounts = budy.BudyAccount.find(**object)

        accounts_s = [("username", "email", "first_name", "last_name", "birth_date")]
        for account in accounts:
            account_s = (
                account.username,
                account.email,
                account.first_name,
                account.last_name,
                account.birth_date,
            )
            accounts_s.append(account_s)

        result = appier.serialize_csv(accounts_s, delimiter=",")
        self.content_type("text/csv")
        return result



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Budy
# Copyright (c) 2008-2024 Hive Solutions Lda.
#
# This file is part of Hive Budy.
#
# Hive Budy is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Budy is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Budy. If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__copyright__ = "Copyright (c) 2008-2024 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

import commons
import logging
import unittest

import appier

import budy


class WishlistTest(unittest.TestCase):
    def setUp(self):
        self.app = budy.BudyApp(level=logging.ERROR)

    def tearDown(self):
        self.app.unload()
        adapter = appier.get_adapter()
        adapter.drop_db()

    def test_basic(self):
        product = budy.Product(short_description="product", gender="Male", price=10.0)
        product.save()

        self.assertEqual(product.short_description, "product")
        self.assertEqual(product.gender, "Male")
        self.assertEqual(product.price, 10.0)

        wishlist = budy.Wishlist()
        wishlist.save()

        self.assertEqual(type(wishlist.key), appier.legacy.UNICODE)
        self.assertEqual(type(wishlist.total), commons.Decimal)
        self.assertEqual(len(wishlist.lines), 0)
        self.assertEqual(wishlist.currency, None)
        self.assertEqual(wishlist.total >= 0.0, True)

        wishlist_line = budy.WishlistLine(quantity=2.0)
        wishlist_line.product = product
        wishlist_line.save()
        wishlist.add_line_s(wishlist_line)

        self.assertEqual(wishlist_line.quantity, 2.0)
        self.assertEqual(wishlist_line.total, 20.0)
        self.assertEqual(wishlist.total, 20.0)
        self.assertEqual(len(wishlist.lines), 1)

        wishlist.add_product_s(product, 3.0)

        self.assertEqual(wishlist.total, 50.0)
        self.assertEqual(len(wishlist.lines), 1)

        product_expensive = budy.Product(
            short_description="product_expensive", gender="Female", price=100.0
        )
        product_expensive.save()

        self.assertEqual(product_expensive.short_description, "product_expensive")
        self.assertEqual(product_expensive.gender, "Female")
        self.assertEqual(product_expensive.price, 100.0)

        wishlist.add_product_s(product_expensive, 1.0)

        self.assertEqual(wishlist.total, 150.0)
        self.assertEqual(len(wishlist.lines), 2)

        wishlist.empty_s()

        self.assertEqual(wishlist.total, 0.0)
        self.assertEqual(len(wishlist.lines), 0)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Budy
# Copyright (c) 2008-2024 Hive Solutions Lda.
#
# This file is part of Hive Budy.
#
# Hive Budy is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Budy is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Budy. If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__copyright__ = "Copyright (c) 2008-2024 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

import time
import commons
import logging
import unittest

import appier

import budy


class VoucherTest(unittest.TestCase):
    def setUp(self):
        self.app = budy.BudyApp(level=logging.ERROR)

    def tearDown(self):
        self.app.unload()
        adapter = appier.get_adapter()
        adapter.drop_db()

    def test_basic(self):
        voucher = budy.Voucher(amount=200.0)
        voucher.save()

        self.assertEqual(voucher.used, False)
        self.assertEqual(voucher.amount, 200.0)
        self.assertEqual(voucher.used_amount, 0.0)
        self.assertEqual(voucher.usage_count, 0)
        self.assertEqual(voucher.is_percent, False)
        self.assertEqual(voucher.is_valid(), True)
        self.assertEqual(type(voucher.key), appier.legacy.UNICODE)
        self.assertNotEqual(voucher.key, None)

        voucher.use_s(100.0)

        self.assertEqual(voucher.is_valid(), True)
        self.assertEqual(voucher.is_valid(amount=100.0), True)
        self.assertEqual(voucher.is_valid(amount=200.0), False)
        self.assertEqual(voucher.used, False)
        self.assertEqual(voucher.amount, 200.0)
        self.assertEqual(voucher.open_amount, 100.0)
        self.assertEqual(voucher.used_amount, 100.0)
        self.assertEqual(voucher.usage_count, 1)
        self.assertEqual(voucher.is_percent, False)
        self.assertEqual(isinstance(voucher.amount, commons.Decimal), True)
        self.assertEqual(isinstance(voucher.open_amount, commons.Decimal), True)

        voucher.use_s(100.0)

        self.assertEqual(voucher.is_valid(), False)
        self.assertEqual(voucher.used, True)
        self.assertEqual(voucher.amount, 200.0)
        self.assertEqual(voucher.open_amount, 0.0)
        self.assertEqual(voucher.used_amount, 200.0)
        self.assertEqual(voucher.usage_count, 2)
        self.assertEqual(voucher.is_percent, False)

        self.assertRaises(
            appier.AssertionError, lambda: voucher.use_s(100.0, currency="EUR")
        )

        budy.Currency.create_s("EUR", 2)
        budy.Currency.create_s("USD", 2)
        budy.ExchangeRate.create_both_s("EUR", "USD", 1.135)

        voucher = budy.Voucher(amount=200.0, currency="EUR")
        voucher.use_s(100.0, currency="EUR")

        self.assertEqual(voucher.is_valid(), True)
        self.assertEqual(voucher.used, False)
        self.assertEqual(voucher.amount, 200.0)
        self.assertEqual(voucher.currency, "EUR")
        self.assertEqual(voucher.open_amount, 100.0)
        self.assertEqual(voucher.used_amount, 100.0)
        self.assertEqual(voucher.usage_count, 1)
        self.assertEqual(voucher.is_percent, False)

        voucher.use_s(100.0, currency="USD")

        self.assertEqual(voucher.is_valid(), True)
        self.assertEqual(voucher.used, False)
        self.assertEqual(voucher.amount, 200.0)
        self.assertEqual(voucher.currency, "EUR")
        self.assertEqual(voucher.open_amount, 11.90)
        self.assertEqual(voucher.used_amount, 188.10)
        self.assertEqual(voucher.usage_count, 2)
        self.assertEqual(voucher.is_percent, False)

        value = voucher.open_amount_r(currency="USD")
        voucher.use_s(value, currency="USD")

        self.assertEqual(voucher.is_valid(), True)
        self.assertEqual(voucher.used, False)
        self.assertEqual(voucher.amount, 200.0)
        self.assertEqual(voucher.currency, "EUR")
        self.assertEqual(voucher.open_amount, 0.01)
        self.assertEqual(voucher.used_amount, 199.99)
        self.assertEqual(voucher.usage_count, 3)
        self.assertEqual(voucher.is_percent, False)

        voucher.use_s(voucher.open_amount, currency="EUR")

        self.assertEqual(voucher.is_valid(), False)
        self.assertEqual(voucher.used, True)
        self.assertEqual(voucher.amount, 200.0)
        self.assertEqual(voucher.currency, "EUR")
        self.assertEqual(voucher.open_amount, 0.0)
        self.assertEqual(voucher.used_amount, 200.0)
        self.assertEqual(voucher.usage_count, 4)
        self.assertEqual(voucher.is_percent, False)

        voucher = budy.Voucher(amount=200.0, currency="EUR", unlimited=True)
        voucher.use_s(100.0, currency="EUR")

        self.assertEqual(voucher.is_valid(), True)
        self.assertEqual(voucher.used, False)
        self.assertEqual(voucher.amount, 200.0)
        self.assertEqual(voucher.currency, "EUR")
        self.assertEqual(voucher.open_amount, 200.0)
        self.assertEqual(voucher.used_amount, 0.0)
        self.assertEqual(voucher.usage_count, 1)
        self.assertEqual(voucher.is_percent, False)
        self.assertEqual(voucher.is_unlimited, True)

        voucher.use_s(100.0, currency="USD")

        self.assertEqual(voucher.is_valid(), True)
        self.assertEqual(voucher.used, False)
        self.assertEqual(voucher.amount, 200.0)
        self.assertEqual(voucher.currency, "EUR")
        self.assertEqual(voucher.open_amount, 200.0)
        self.assertEqual(voucher.used_amount, 0.0)
        self.assertEqual(voucher.usage_count, 2)
        self.assertEqual(voucher.is_percent, False)
        self.assertEqual(voucher.is_unlimited, True)

    def test_single(self):
        voucher = budy.Voucher(amount=200.0, usage_limit=1)
        voucher.save()

        voucher.use_s(100.0)

        self.assertEqual(voucher.is_valid(), False)
        self.assertEqual(voucher.is_valid(amount=100.0), False)
        self.assertEqual(voucher.is_valid(amount=200.0), False)
        self.assertEqual(voucher.used, True)
        self.assertEqual(voucher.amount, 200.0)
        self.assertEqual(voucher.open_amount, 100.0)
        self.assertEqual(voucher.used_amount, 100.0)
        self.assertEqual(voucher.usage_count, 1)
        self.assertEqual(voucher.is_percent, False)
        self.assertEqual(isinstance(voucher.amount, commons.Decimal), True)
        self.assertEqual(isinstance(voucher.open_amount, commons.Decimal), True)

        self.assertRaises(appier.AssertionError, lambda: voucher.use_s(100.0))

    def test_percent(self):
        budy.Currency.create_s("EUR", 2)

        voucher = budy.Voucher(percentage=10.0)
        voucher.save()

        self.assertEqual(voucher.open_amount_p(100.0), 10.0)
        self.assertEqual(voucher.discount(100.0), 10.0)
        self.assertEqual(voucher.discount(100.234, currency="EUR"), 10.02)

        voucher = budy.Voucher(percentage=120.0)
        self.assertRaises(appier.ValidationError, voucher.save)

    def test_unstarted(self):
        voucher = budy.Voucher(amount=200.0, start=time.time() - 3600)
        voucher.save()

        voucher.use_s(100.0)

        self.assertEqual(voucher.is_valid(), True)
        self.assertEqual(voucher.is_valid(amount=100.0), True)
        self.assertEqual(voucher.is_valid(amount=200.0), False)
        self.assertEqual(voucher.used, False)
        self.assertEqual(voucher.amount, 200.0)
        self.assertEqual(voucher.open_amount, 100.0)
        self.assertEqual(voucher.used_amount, 100.0)
        self.assertEqual(voucher.usage_count, 1)
        self.assertEqual(voucher.is_percent, False)
        self.assertEqual(isinstance(voucher.amount, commons.Decimal), True)
        self.assertEqual(isinstance(voucher.open_amount, commons.Decimal), True)

        voucher = budy.Voucher(amount=200.0, start=time.time() + 60)
        voucher.save()

        self.assertEqual(voucher.is_used(), False)
        self.assertEqual(voucher.is_valid(), False)
        self.assertEqual(voucher.is_valid(amount=100.0), False)
        self.assertEqual(voucher.is_valid(amount=200.0), False)
        self.assertEqual(voucher.used, False)

        self.assertRaises(appier.AssertionError, lambda: voucher.use_s(100.0))

    def test_expired(self):
        voucher = budy.Voucher(amount=200.0, expiration=time.time() + 3600)
        voucher.save()

        voucher.use_s(100.0)

        self.assertEqual(voucher.is_valid(), True)
        self.assertEqual(voucher.is_valid(amount=100.0), True)
        self.assertEqual(voucher.is_valid(amount=200.0), False)
        self.assertEqual(voucher.used, False)
        self.assertEqual(voucher.amount, 200.0)
        self.assertEqual(voucher.open_amount, 100.0)
        self.assertEqual(voucher.used_amount, 100.0)
        self.assertEqual(voucher.usage_count, 1)
        self.assertEqual(voucher.is_percent, False)
        self.assertEqual(isinstance(voucher.amount, commons.Decimal), True)
        self.assertEqual(isinstance(voucher.open_amount, commons.Decimal), True)

        voucher = budy.Voucher(amount=200.0, expiration=time.time() - 60)
        voucher.save()

        self.assertEqual(voucher.is_used(), False)
        self.assertEqual(voucher.is_valid(), False)
        self.assertEqual(voucher.is_valid(amount=100.0), False)
        self.assertEqual(voucher.is_valid(amount=200.0), False)
        self.assertEqual(voucher.used, False)

        self.assertRaises(appier.AssertionError, lambda: voucher.use_s(100.0))

    def test_disuse(self):
        voucher = budy.Voucher(amount=200.0)
        voucher.save()

        voucher.use_s(100.0)

        self.assertEqual(voucher.is_valid(), True)
        self.assertEqual(voucher.is_valid(amount=100.0), True)
        self.assertEqual(voucher.is_valid(amount=200.0), False)
        self.assertEqual(voucher.used, False)
        self.assertEqual(voucher.amount, 200.0)
        self.assertEqual(voucher.open_amount, 100.0)
        self.assertEqual(voucher.used_amount, 100.0)
        self.assertEqual(voucher.usage_count, 1)
        self.assertEqual(voucher.is_percent, False)
        self.assertEqual(isinstance(voucher.amount, commons.Decimal), True)
        self.assertEqual(isinstance(voucher.open_amount, commons.Decimal), True)

        voucher.use_s(100.0)

        self.assertEqual(voucher.is_valid(), False)
        self.assertEqual(voucher.is_valid(amount=100.0), False)
        self.assertEqual(voucher.is_valid(amount=200.0), False)
        self.assertEqual(voucher.used, True)
        self.assertEqual(voucher.amount, 200.0)
        self.assertEqual(voucher.open_amount, 0.0)
        self.assertEqual(voucher.used_amount, 200.0)
        self.assertEqual(voucher.usage_count, 2)
        self.assertEqual(voucher.is_percent, False)
        self.assertEqual(isinstance(voucher.amount, commons.Decimal), True)
        self.assertEqual(isinstance(voucher.open_amount, commons.Decimal), True)

        voucher.disuse_s(100.0)

        self.assertEqual(voucher.is_valid(), True)
        self.assertEqual(voucher.is_valid(amount=100.0), True)
        self.assertEqual(voucher.is_valid(amount=200.0), False)
        self.assertEqual(voucher.used, False)
        self.assertEqual(voucher.amount, 200.0)
        self.assertEqual(voucher.open_amount, 100.0)
        self.assertEqual(voucher.used_amount, 100.0)
        self.assertEqual(voucher.usage_count, 1)
        self.assertEqual(voucher.is_percent, False)
        self.assertEqual(isinstance(voucher.amount, commons.Decimal), True)
        self.assertEqual(isinstance(voucher.open_amount, commons.Decimal), True)

        voucher.disuse_s(100.0)

        self.assertEqual(voucher.is_valid(), True)
        self.assertEqual(voucher.is_valid(amount=100.0), True)
        self.assertEqual(voucher.is_valid(amount=200.0), True)
        self.assertEqual(voucher.used, False)
        self.assertEqual(voucher.amount, 200.0)
        self.assertEqual(voucher.open_amount, 200.0)
        self.assertEqual(voucher.used_amount, 0.0)
        self.assertEqual(voucher.usage_count, 0)
        self.assertEqual(voucher.is_percent, False)
        self.assertEqual(isinstance(voucher.amount, commons.Decimal), True)
        self.assertEqual(isinstance(voucher.open_amount, commons.Decimal), True)

        self.assertRaises(appier.AssertionError, lambda: voucher.disuse_s(1.0))

        voucher = budy.Voucher(percentage=10.0)
        voucher.save()

        self.assertEqual(voucher.open_amount_p(100.0), 10.0)
        self.assertEqual(voucher.used, False)
        self.assertEqual(voucher.usage_count, 0)
        self.assertEqual(voucher.is_percent, True)
        self.assertEqual(isinstance(voucher.amount, commons.Decimal), True)
        self.assertEqual(isinstance(voucher.open_amount, commons.Decimal), True)

        voucher.use_s(100.0)

        self.assertEqual(voucher.open_amount_p(100.0), 10.0)
        self.assertEqual(voucher.used, False)
        self.assertEqual(voucher.usage_count, 1)
        self.assertEqual(voucher.is_percent, True)
        self.assertEqual(isinstance(voucher.amount, commons.Decimal), True)
        self.assertEqual(isinstance(voucher.open_amount, commons.Decimal), True)

        voucher.disuse_s(100.0)

        self.assertEqual(voucher.open_amount_p(100.0), 10.0)
        self.assertEqual(voucher.used, False)
        self.assertEqual(voucher.usage_count, 0)
        self.assertEqual(voucher.is_percent, True)
        self.assertEqual(isinstance(voucher.amount, commons.Decimal), True)
        self.assertEqual(isinstance(voucher.open_amount, commons.Decimal), True)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Budy
# Copyright (c) 2008-2024 Hive Solutions Lda.
#
# This file is part of Hive Budy.
#
# Hive Budy is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Budy is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Budy. If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__copyright__ = "Copyright (c) 2008-2024 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

import appier
import appier_extras

from . import bundle
from . import wishlist_line


class Wishlist(bundle.Bundle):
    lines = appier.field(type=appier.references("WishlistLine", name="id"), eager=True)

    account = appier.field(type=appier.reference("BudyAccount", name="id"), eager=True)

    @classmethod
    def list_names(cls):
        return ["id", "key", "total", "currency", "account"]

    @classmethod
    def line_cls(cls):
        return wishlist_line.WishlistLine

    @classmethod
    def from_session(cls, ensure=True, raise_e=False):
        from . import BudyAccount

        account = BudyAccount.from_session(raise_e=raise_e)
        if account:
            return account.ensure_wishlist_s()
        session = appier.get_session()
        key = session.get("wishlist", None)
        wishlist = cls.get(key=key, raise_e=raise_e) if key else None
        if wishlist:
            return wishlist
        if not ensure:
            return None
        wishlist = cls()
        wishlist.save()
        session["wishlist"] = wishlist.key
        return wishlist

    @classmethod
    def ensure_s(cls, key=None):
        from . import BudyAccount

        account = BudyAccount.from_session(raise_e=False)
        if account:
            return account.ensure_wishlist_s(key=key)
        wishlist = cls(key=key)
        wishlist.save()
        return wishlist

    def pre_validate(self):
        bundle.Bundle.pre_validate(self)
        self.try_valid()

    def pre_delete(self):
        bundle.Bundle.pre_delete(self)
        for line in self.lines:
            line.delete()

    def add_line_s(self, line):
        line.wishlist = self
        return bundle.Bundle.add_line_s(self, line)

    @appier.operation(name="Garbage Collect")
    def collect_s(self):
        self.delete()

    @appier.operation(name="Fix Orphans")
    def fix_orphans_s(self):
        for line in self.lines:
            line.wishlist = self
            line.save()

    @appier.operation(name="Notify")
    def notify(self, name=None, *args, **kwargs):
        name = name or "wishlist.new"
        wishlist = self.reload(map=True)
        account = wishlist.get("account", {})
        account = kwargs.get("account", account)
        receiver = account.get("email", None)
        receiver = kwargs.get("email", receiver)
        appier_extras.admin.Event.notify_g(
            name,
            arguments=dict(
                params=dict(
                    payload=wishlist, wishlist=wishlist, receiver=receiver, extra=kwargs
                )
            ),
        )

    @appier.operation(name="Remind")
    def remind(self):
        self.notify(name="wishlist.remind")



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Budy
# Copyright (c) 2008-2024 Hive Solutions Lda.
#
# This file is part of Hive Budy.
#
# Hive Budy is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Budy is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Budy. If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__copyright__ = "Copyright (c) 2008-2024 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

import appier

import budy

from . import root


class AddressAPIController(root.RootAPIController):
    @appier.route("/api/addresses", "GET", json=True)
    @appier.ensure(token="admin")
    def list(self):
        object = appier.get_object(alias=True, find=True)
        addresses = budy.Address.find(map=True, **object)
        return addresses

    @appier.route("/api/addresses", "POST", json=True)
    @appier.ensure(token="user")
    def create(self):
        address = budy.Address.new()
        address.save()
        address = address.map()
        return address

    @appier.route("/api/addresses/<str:key>", "GET", json=True)
    @appier.ensure(token="user")
    def show(self, key):
        address = budy.Address.get(key=key, map=True)
        return address

    @appier.route("/api/addresses/<str:key>", "PUT", json=True)
    @appier.ensure(token="user")
    def update(self, key):
        address = budy.Address.get(key=key, rules=False)
        address.apply()
        address.save()
        address = address.map()
        return address

    @appier.route("/api/addresses/<str:key>", "DELETE", json=True)
    def delete(self, key):
        address = budy.Address.get(key=key)
        address.delete()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Budy
# Copyright (c) 2008-2024 Hive Solutions Lda.
#
# This file is part of Hive Budy.
#
# Hive Budy is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Budy is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Budy. If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__copyright__ = "Copyright (c) 2008-2024 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

import appier

from . import base


class Currency(base.BudyBase):
    iso = appier.field(default=True, index=True, description="ISO")

    decimal_places = appier.field(type=int, index=True)

    @classmethod
    def teardown(cls):
        super(Currency, cls).teardown()
        cls.invalidate()

    @classmethod
    def validate(cls):
        return super(Currency, cls).validate() + [
            appier.not_null("iso"),
            appier.not_empty("iso"),
            appier.string_eq("iso", 3),
            appier.not_null("decimal_places"),
            appier.gte("decimal_places", 0),
        ]

    @classmethod
    def list_names(cls):
        return ["iso", "decimal_places"]

    @classmethod
    def create_s(cls, iso, decimal_places, invalidate=True):
        if invalidate:
            cls.invalidate()
        currency = cls(iso=iso, decimal_places=decimal_places)
        currency.save()

    @classmethod
    def round(cls, value, currency, rounder=round, decimal_places=5):
        currencies = cls.get_currencies()
        currency = currencies.get(currency, {})
        decimal_places = currency.get("decimal_places", decimal_places)
        return rounder(value, decimal_places)

    @classmethod
    def format(cls, value, currency, decimal_places=2):
        currencies = cls.get_currencies()
        currency = currencies.get(currency, {})
        decimal_places = currency.get("decimal_places", decimal_places)
        format = "%%.%df" % decimal_places
        return format % value

    @classmethod
    def get_currencies(cls, app=None):
        app = app or appier.get_app()
        if hasattr(app, "_currencies"):
            return app._currencies
        currencies = cls.find(map=True)
        app._currencies = dict([(value["iso"], value) for value in currencies])
        return app._currencies

    @classmethod
    def invalidate(cls, app=None):
        app = app or appier.get_app()
        if not hasattr(app, "_currencies"):
            return
        delattr(app, "_currencies")

    @classmethod
    @appier.operation(
        name="Import CSV",
        parameters=(
            ("CSV File", "file", "file"),
            ("Empty source", "empty", bool, False),
        ),
    )
    def import_csv_s(cls, file, empty):
        def callback(line):
            iso, decimal_places = line
            decimal_places = int(decimal_places)
            currency = cls(iso=iso, decimal_places=decimal_places)
            currency.save()

        if empty:
            cls.delete_c()
        cls._csv_import(file, callback)

    @classmethod
    @appier.link(name="Export Simple")
    def simple_csv_url(cls, absolute=False):
        return appier.get_app().url_for("currency_api.simple_csv", absolute=absolute)

    @classmethod
    def _plural(cls):
        return "Currencies"



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Budy
# Copyright (c) 2008-2024 Hive Solutions Lda.
#
# This file is part of Hive Budy.
#
# Hive Budy is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Budy is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Budy. If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__copyright__ = "Copyright (c) 2008-2024 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

import json
import commons

import appier

from . import base


class BundleLine(base.BudyBase):
    price = appier.field(
        type=commons.Decimal,
        observations="""The unit price for the item described by the
        current line, the total should be calculated using this value""",
    )

    taxes = appier.field(
        type=commons.Decimal,
        observations="""The amount of taxes paid per each item in the
        current line, this value is already included in the price value
        of the line""",
    )

    currency = appier.field(
        observations="""The currency in use for the line all the price
        related values for the line are considered to be represented
        under this given currency"""
    )

    country = appier.field()

    quantity = appier.field(type=commons.Decimal, initial=commons.Decimal(0.0))

    total = appier.field(type=commons.Decimal, initial=commons.Decimal(0.0))

    total_taxes = appier.field(type=commons.Decimal, initial=commons.Decimal(0.0))

    size = appier.field(type=int)

    size_s = appier.field()

    scale = appier.field(type=int)

    discounted = appier.field(
        type=bool,
        initial=False,
        index=True,
        observations="""If the product associated with the
        current line is a discounted one, meaning that the product
        price is affected by some discount""",
    )

    closed = appier.field(
        type=bool,
        initial=False,
        index=True,
        observations="""Simple flag that control if the line is
        closed meaning that calculated values (eg: totals) can no
        longer be re-calculated as the line is now frozen""",
    )

    attributes = appier.field(
        observations="""Additional metadata that define the given
        (bundle) line, this is typically used to store a JSON serialized
        string with extra fields"""
    )

    product = appier.field(
        type=appier.reference("Product", name="id"),
        eager=True,
        observations="""The product that this bundle line is
        associated with, this is the product that is going to
        be used for the calculation of the price""",
    )

    @classmethod
    def list_names(cls):
        return ["id", "quantity", "total", "currency", "product"]

    @classmethod
    def order_name(cls):
        return ["id", -1]

    @classmethod
    def is_abstract(cls):
        return True

    def pre_save(self):
        base.BudyBase.pre_save(self)
        self.calculate()
        self.measure()
        self.ensure_valid()

    def calculate(self, currency=None, country=None, force=False):
        if self.closed:
            return
        currency = currency or self.currency
        country = country or self.country
        self.total_taxes = self.quantity * self.get_taxes(
            currency=currency, country=country, force=force
        )
        self.total = self.quantity * self.get_price(
            currency=currency, country=country, force=force
        )
        self.discounted = self.merchandise.is_discounted

    def measure(self, currency=None, country=None, force=False):
        if self.closed:
            return
        if self.size and self.scale and not force:
            return
        self.size, self.scale = self.get_size(
            currency=currency, country=country, force=force
        )

    def close_s(self):
        self.closed = True
        self.save()

    def get_price(self, currency=None, country=None, force=False):
        is_dirty = self.is_dirty(currency=currency, country=country)
        if not is_dirty and not force:
            return self.price
        self.price = self.merchandise.get_price(
            currency=currency, country=country, attributes=self.attributes
        )
        self.taxes = self.merchandise.get_taxes(
            currency=currency, country=country, attributes=self.attributes
        )
        self.currency = self.merchandise.get_currency(currency=currency)
        self.country = country
        return self.price

    def get_taxes(self, currency=None, country=None, force=False):
        self.get_price(currency=currency, country=country, force=force)
        return self.taxes

    def get_size(self, currency=None, country=None, force=False):
        if not self.product:
            return None, None
        return self.product.get_size(
            currency=currency, country=country, attributes=self.attributes
        )

    def ensure_size_s(self):
        if not self.size:
            return
        if not hasattr(self.merchandise, "value_s"):
            return
        self.size_s = self.merchandise.value_s

    def ensure_valid(self):
        appier.verify(self.is_valid())

    def try_valid(self):
        fixed = False
        fixed |= self.try_valid_quantity()
        fixed |= self.try_valid_price()
        return fixed

    def try_valid_s(self):
        fixed = self.try_valid()
        if not fixed:
            return fixed
        self.save()
        return fixed

    def try_valid_quantity(self):
        fixed = False
        if self.quantity <= 0.0:
            self.quantity = 0.0
        if self.merchandise.quantity_hand == None:
            return fixed
        if self.quantity <= self.merchandise.quantity_hand:
            return fixed
        self.quantity = min(self.quantity, self.merchandise.quantity_hand)
        self.calculate(force=True)
        fixed |= True
        return fixed

    def try_valid_price(self):
        fixed = False
        if self.merchandise.is_price_provided:
            return fixed
        if self.merchandise.price == None:
            return fixed
        if self.merchandise.price == self.price:
            return fixed
        self.price = self.merchandise.price
        self.calculate(force=True)
        fixed |= True
        return fixed

    def is_empty(self):
        return self.quantity == 0.0

    def is_dirty(self, currency=None, country=None):
        is_dirty = not self.currency == currency
        is_dirty |= not self.country == country
        is_dirty |= not hasattr(self, "price") or self.price == None
        is_dirty |= not hasattr(self, "taxes") or self.taxes == None
        return is_dirty

    def is_valid(self):
        is_valid = self.is_valid_quantity()
        is_valid &= self.is_valid_price()
        is_valid &= self.is_valid_size()
        return is_valid

    def is_discountable(self, strict=False):
        if not self.merchandise.is_discountable:
            return False
        if strict and self.discounted:
            return False
        return True

    def is_valid_quantity(self, reload=True):
        if self.quantity < 0:
            return False
        merchandise = (
            self.merchandise and self.merchandise.reload()
            if reload
            else self.merchandise
        )
        if (
            not merchandise.quantity_hand == None
            and self.quantity > merchandise.quantity_hand
        ):
            return False
        return True

    def is_valid_price(self, reload=True):
        merchandise = (
            self.merchandise and self.merchandise.reload()
            if reload
            else self.merchandise
        )
        if (
            not merchandise.is_price_provided
            and not merchandise.price == None
            and not self.price == merchandise.price
        ):
            return False
        return True

    def is_valid_size(self, reload=True):
        if not self.product.is_parent:
            return True
        return True if self.size else False

    @appier.operation(name="Calculate")
    def calculate_s(self):
        self.calculate(force=True)
        self.save()

    @appier.operation(name="Measure")
    def measure_s(self):
        self.measure(force=True)
        self.save()

    @appier.operation(name="Recover product")
    def recover_s(self):
        from . import product

        self.product.resolve()
        if self.product.is_resolved():
            return

        if not self.attributes:
            return
        attributes = json.loads(self.attributes)

        product_id = attributes.get("product_id", None)
        if not product_id:
            return
        product_id_s = str(product_id)

        _product = product.Product.get(product_id=product_id_s)
        if not _product:
            return

        self.product = _product
        self.save(validate=False, pre_save=False)

    @property
    def merchandise(self, name="size", strict=False):
        if not self.size:
            return self.product
        if not hasattr(self.product, "get_measurement"):
            return self.product
        measurement = self.product.get_measurement(self.size, name=name)
        if measurement or strict:
            return measurement
        return self.product



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Budy
# Copyright (c) 2008-2024 Hive Solutions Lda.
#
# This file is part of Hive Budy.
#
# Hive Budy is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Budy is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Budy. If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__copyright__ = "Copyright (c) 2008-2024 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

import commons

import appier

from . import base


class ExchangeRate(base.BudyBase):
    name = appier.field(default=True)

    base = appier.field(index=True)

    target = appier.field(index=True)

    rate = appier.field(type=commons.Decimal, index=True)

    @classmethod
    def validate(cls):
        return super(ExchangeRate, cls).validate() + [
            appier.not_null("base"),
            appier.not_empty("base"),
            appier.not_null("target"),
            appier.not_empty("target"),
            appier.not_null("rate"),
            appier.gte("rate", 0.0),
        ]

    @classmethod
    def list_names(cls):
        return ["base", "target", "rate"]

    @classmethod
    def create_s(cls, base, target, rate):
        exchange_rate = cls(base=base, target=target, rate=rate)
        exchange_rate.save()

    @classmethod
    def create_both_s(cls, base, target, rate):
        rate_r = commons.Decimal(1.0) / rate
        cls.create_s(base, target, rate)
        cls.create_s(target, base, rate_r)

    @classmethod
    def convert(cls, value, base, target, reversed=False, rounder=round):
        from . import currency

        if reversed:
            return cls.reverse(value, base, target, rounder=rounder)
        exchange_rate = cls.get(base=base, target=target)
        result = commons.Decimal(value) * exchange_rate.rate
        return currency.Currency.round(result, target, rounder=rounder)

    @classmethod
    def reverse(cls, value, base, target, rounder=round):
        from . import currency

        exchange_rate = cls.get(base=target, target=base)
        result = commons.Decimal(value) * (commons.Decimal(1.0) / exchange_rate.rate)
        return currency.Currency.round(result, target, rounder=rounder)

    @classmethod
    def has_rate(cls, base, target):
        exchange_rate = cls.get(base=base, target=target, raise_e=False)
        if not exchange_rate:
            return False
        if not exchange_rate.rate:
            return False
        return True

    @classmethod
    @appier.operation(
        name="Import CSV",
        parameters=(
            ("CSV File", "file", "file"),
            ("Empty source", "empty", bool, False),
        ),
    )
    def import_csv_s(cls, file, empty):
        def callback(line):
            base, target, rate = line
            rate = float(rate)
            exchange_rate = cls(base=base, target=target, rate=rate)
            exchange_rate.save()

        if empty:
            cls.delete_c()
        cls._csv_import(file, callback)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Budy
# Copyright (c) 2008-2024 Hive Solutions Lda.
#
# This file is part of Hive Budy.
#
# Hive Budy is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Budy is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Budy. If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__copyright__ = "Copyright (c) 2008-2024 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

import appier

import budy

from . import root


class CountryAPIController(root.RootAPIController):
    @appier.route("/api/countries", "GET", json=True)
    def list(self):
        object = appier.get_object(alias=True, find=True)
        countries = budy.Country.find(find_i=True, find_t="right", map=True, **object)
        return countries

    @appier.route("/api/countries/simple.csv", "GET")
    @appier.ensure(token="admin")
    def simple_csv(self):
        object = appier.get_object(alias=True, find=True, limit=0)
        countries = budy.Country.find(**object)

        countries_s = [("name", "country_code", "currency_code", "locale")]
        for country in countries:
            country_s = (
                country.name,
                country.country_code,
                country.currency_code,
                country.locale,
            )
            countries_s.append(country_s)

        result = appier.serialize_csv(countries_s, delimiter=",")
        self.content_type("text/csv")
        return result



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Budy
# Copyright (c) 2008-2024 Hive Solutions Lda.
#
# This file is part of Hive Budy.
#
# Hive Budy is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Budy is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Budy. If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__copyright__ = "Copyright (c) 2008-2024 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

import os
import zipfile
import tempfile
import mimetypes

import appier

from . import base
from . import product

BASE_URL = "http://localhost:8080/"


class Media(base.BudyBase):
    label = appier.field(index=True)

    order = appier.field(type=int, index=True)

    size = appier.field(index=True)

    unique = appier.field(index=True, safe=True)

    file = appier.field(type=appier.File, private=True)

    @classmethod
    def validate(cls):
        return super(Media, cls).validate() + [
            appier.not_null("description"),
            appier.not_empty("description"),
            appier.not_null("label"),
            appier.not_empty("label"),
            appier.not_null("order"),
            appier.not_null("file"),
            appier.not_empty("file"),
        ]

    @classmethod
    def list_names(cls):
        return ["id", "description", "label", "order", "size"]

    @classmethod
    def order_name(cls):
        return ["id", -1]

    @classmethod
    def is_visible(cls):
        return False

    @classmethod
    @appier.operation(
        name="Import Media",
        parameters=(
            ("Media File", "file", "file"),
            ("Empty source", "empty", bool, False),
        ),
    )
    def import_media_s(cls, file, empty, strict=False):
        _file_name, mime_type, data = file
        is_zip = mime_type in ("application/zip", "application/octet-stream")
        if not is_zip and strict:
            raise appier.OperationalError(message="Invalid MIME type '%s'" % mime_type)
        buffer = appier.legacy.BytesIO(data)
        file = zipfile.ZipFile(buffer, mode="r")
        target = tempfile.mkdtemp()
        try:
            file.extractall(target)
        finally:
            file.close()

        if empty:
            cls.delete_c()

        names = os.listdir(target)
        for name in names:
            path = os.path.join(target, name)
            is_dir = os.path.isdir(path)
            if is_dir:
                image_names = os.listdir(path)
            else:
                path = target
                image_names = [name]
            for image_name in image_names:
                base, extension = os.path.splitext(image_name)
                if not extension in (".png", ".jpeg", ".jpg"):
                    continue

                content_type, _encoding = mimetypes.guess_type(image_name, strict=False)

                product_id = base
                label = "undefined"
                order = 0
                size = "large"

                base_s = base.split("_")
                if len(base_s) >= 1:
                    product_id = base_s[0]
                if len(base_s) >= 2:
                    order = int(base_s[1])
                if len(base_s) >= 3:
                    label = base_s[2]
                if len(base_s) >= 4:
                    size = base_s[3]

                description = "%s_%s_%s" % (product_id, label, size)

                image_path = os.path.join(path, image_name)
                image_file = open(image_path, "rb")
                try:
                    image_data = image_file.read()
                finally:
                    image_file.close()

                media = Media(
                    description=description,
                    label=label,
                    order=order,
                    size=size,
                    file=appier.File((product_id, content_type, image_data)),
                )
                media.save()

                _product = product.Product.get(product_id=product_id, raise_e=False)
                if not _product:
                    continue

                _product.images.append(media)
                _product.save()

    @classmethod
    def _build(cls, model, map):
        super(Media, cls)._build(model, map)
        id = model.get("id", None)
        if id:
            model["url"] = cls._get_url(id)
            for format in ("png", "jpeg", "webp"):
                model["url_" + format] = cls._get_url(id, format=format)

    @classmethod
    def _plural(cls):
        return "Media"

    @classmethod
    def _get_url(cls, id, format=None, absolute=True):
        app = appier.get_app()
        if format:
            return app.url_for(
                "media_api.data_format",
                id=id,
                format=format,
                prefix="/",
                absolute=absolute,
            )
        else:
            return app.url_for("media_api.data", id=id, prefix="/", absolute=absolute)

    def get_url(self, format=None):
        return self.__class__._get_url(self.id, format=format)

    def convert_image(self, format, background=None, **kwargs):
        import PIL.Image

        kwargs = dict(kwargs)
        buffer = appier.legacy.BytesIO()
        image = PIL.Image.open(appier.legacy.BytesIO(self.file.data))
        if format in ("jpeg",) and not background:
            background = "ffffff"
        if background:
            image_background = PIL.Image.new(
                "RGB", (image.width, image.height), color="#" + background
            )
            if image.mode == "RGBA":
                image_background.paste(image, mask=image)
            else:
                image_background.paste(image)
            image = image_background
        image.save(buffer, format=format, **kwargs)
        return buffer

    @appier.operation(
        name="Generate Thumbnail",
        parameters=(
            ("Width", "width", int),
            ("Height", "height", int),
            ("Format", "format", str, "png"),
            ("Suffix", "suffix", str, "thumbnail"),
        ),
        factory=True,
    )
    def thumbnail_s(self, width=None, height=None, format="png", suffix="thumbnail"):
        cls = self.__class__
        media = self.reload(rules=False)
        builder = appier.image(
            width=width or height, height=height or width, format=format
        )
        image = builder(media.file)
        data = image.resize()
        name = "%s.%s" % (suffix, format)
        mime, _encoding = mimetypes.guess_type(name, strict=False)
        thumbnail = cls(
            description=media.description,
            label=suffix,
            order=media.order,
            size=suffix,
            unique="%s-%s" % (media.unique, suffix),
            file=appier.File((name, mime, data)),
        )
        thumbnail.save()
        return thumbnail

    @appier.link(name="View")
    def view_url(self, absolute=False):
        return self.owner.url_for(
            "media_api.data", id=self.id, prefix="/", absolute=absolute
        )



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Budy
# Copyright (c) 2008-2024 Hive Solutions Lda.
#
# This file is part of Hive Budy.
#
# Hive Budy is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Budy is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Budy. If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__copyright__ = "Copyright (c) 2008-2024 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

import appier
import appier_extras

from . import order
from . import bundle
from . import bag_line


class Bag(bundle.Bundle):
    lines = appier.field(type=appier.references("BagLine", name="id"), eager=True)

    account = appier.field(
        type=appier.reference("BudyAccount", name="id"), index="hashed", eager=True
    )

    @classmethod
    def list_names(cls):
        return ["id", "key", "total", "currency", "account"]

    @classmethod
    def line_cls(cls):
        return bag_line.BagLine

    @classmethod
    def from_session(cls, ensure=True, raise_e=False):
        from . import BudyAccount

        account = BudyAccount.from_session(raise_e=raise_e)
        if account:
            return account.ensure_bag_s()
        session = appier.get_session()
        key = session.get("bag", None)
        bag = cls.get(key=key, raise_e=raise_e) if key else None
        if bag:
            return bag
        if not ensure:
            return None
        bag = cls()
        bag.save()
        session["bag"] = bag.key
        return bag

    @classmethod
    def ensure_s(cls, key=None):
        from . import BudyAccount

        account = BudyAccount.from_session(raise_e=False)
        if account:
            return account.ensure_bag_s(key=key)
        bag = cls(key=key)
        bag.save()
        return bag

    def pre_validate(self):
        bundle.Bundle.pre_validate(self)
        self.try_valid()

    def pre_delete(self):
        bundle.Bundle.pre_delete(self)
        for line in self.lines:
            line.delete()

    def add_line_s(self, line):
        line.bag = self
        return bundle.Bundle.add_line_s(self, line)

    def to_order_s(self, verify=True, try_valid=True):
        self.refresh_s()
        if try_valid:
            self.try_valid_s()
        if verify:
            self.verify_order()
        _order = order.Order(
            currency=self.currency,
            country=self.country,
            sub_total=self.sub_total,
            discounted_sub_total=self.discounted_sub_total,
            undiscounted_sub_total=self.undiscounted_sub_total,
            discount=self.discount,
            shipping_cost=self.shipping_cost,
            taxes=self.taxes,
            total=self.total,
            account=self.account,
            store=self.account and self.account.store,
        )
        _order.save()
        _order.lines = []
        for line in self.lines:
            order_line = line.to_order_line_s(_order)
            _order.lines.append(order_line)
        _order.save()
        if verify:
            _order.verify_base()
        return _order

    def verify_order(self):
        """
        Runs a series of verifications to make sure that the bag
        is ready to be used to generate/create an order.
        """

        appier.verify(len(self.lines) > 0)
        for line in self.lines:
            appier.verify(line.is_valid())

    @appier.operation(name="Garbage Collect")
    def collect_s(self):
        self.delete()

    @appier.operation(name="Fix Orphans")
    def fix_orphans_s(self):
        for line in self.lines:
            line.bag = self
            line.save()

    @appier.operation(name="Notify")
    def notify(self, name=None, *args, **kwargs):
        name = name or "bag.new"
        bag = self.reload(map=True)
        account = bag.get("account", {})
        account = kwargs.get("email", account)
        receiver = account.get("email", None)
        receiver = kwargs.get("email", receiver)
        appier_extras.admin.Event.notify_g(
            name,
            arguments=dict(
                params=dict(payload=bag, bag=bag, receiver=receiver, extra=kwargs)
            ),
        )

    @appier.operation(name="Remind")
    def remind(self, *args, **kwargs):
        self.notify(name="bag.remind", *args, **kwargs)

    @appier.view(name="Lines")
    def lines_v(self, *args, **kwargs):
        kwargs["sort"] = kwargs.get("sort", [("created", -1)])
        return appier.lazy_dict(
            model=self.lines._target,
            kwargs=kwargs,
            entities=appier.lazy(lambda: self.lines.find(*args, **kwargs)),
            page=appier.lazy(lambda: self.lines.paginate(*args, **kwargs)),
            names=["product", "quantity", "total", "currency"],
        )



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Budy
# Copyright (c) 2008-2024 Hive Solutions Lda.
#
# This file is part of Hive Budy.
#
# Hive Budy is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Budy is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Budy. If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__copyright__ = "Copyright (c) 2008-2024 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

import appier

from . import base


class Country(base.BudyBase):
    name = appier.field(index=True, default=True)

    country_code = appier.field(index=True)

    currency_code = appier.field(index=True)

    locale = appier.field(index=True)

    @classmethod
    def validate(cls):
        return super(Country, cls).validate() + [
            appier.not_null("name"),
            appier.not_empty("name"),
            appier.not_null("country_code"),
            appier.not_empty("country_code"),
            appier.string_eq("country_code", 2),
            appier.not_null("currency_code"),
            appier.not_empty("currency_code"),
            appier.string_eq("currency_code", 3),
            appier.not_null("locale"),
            appier.not_empty("locale"),
            appier.string_eq("locale", 5),
        ]

    @classmethod
    def list_names(cls):
        return ["id", "name", "country_code", "currency_code", "locale"]

    @classmethod
    def get_by_code(cls, country_code, *args, **kwargs):
        return cls.get(country_code=country_code, *args, **kwargs)

    @classmethod
    @appier.operation(
        name="Import CSV",
        parameters=(
            ("CSV File", "file", "file"),
            ("Empty source", "empty", bool, False),
        ),
    )
    def import_csv_s(cls, file, empty):
        def callback(line):
            name, country_code, currency_code, locale = line
            name = name or None
            country_code = country_code or None
            currency_code = currency_code or None
            locale = locale or None
            country = cls(
                name=name,
                country_code=country_code,
                currency_code=currency_code,
                locale=locale,
            )
            country.save()

        if empty:
            cls.delete_c()
        cls._csv_import(file, callback)

    @classmethod
    @appier.link(name="Export Simple")
    def simple_csv_url(cls, absolute=False):
        return appier.get_app().url_for("country_api.simple_csv", absolute=absolute)

    @classmethod
    def _plural(cls):
        return "Countries"



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Budy
# Copyright (c) 2008-2024 Hive Solutions Lda.
#
# This file is part of Hive Budy.
#
# Hive Budy is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Budy is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Budy. If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__copyright__ = "Copyright (c) 2008-2024 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

import appier

import budy

from . import root


class BagAPIController(root.RootAPIController):
    @appier.route("/api/bags", "GET", json=True)
    @appier.ensure(token="admin")
    def list(self):
        object = appier.get_object(alias=True, find=True)
        bags = budy.Bag.find(eager_l=True, map=True, **object)
        return bags

    @appier.route("/api/bags", "POST", json=True)
    def create(self):
        bag = budy.Bag.new()
        bag.save()
        bag = bag.map()
        return bag

    @appier.route("/api/bags/key", "GET", json=True)
    def key(self):
        bag = budy.Bag.from_session()
        return dict(key=bag.key)

    @appier.route("/api/bags/<str:key>", "GET", json=True)
    def show(self, key):
        ensure = self.field("ensure", True, cast=bool)
        try_valid = self.field("try_valid", True, cast=bool)
        bag = budy.Bag.get(key=key, raise_e=not ensure)
        if not bag:
            bag = budy.Bag.ensure_s(key=key)
        bag.refresh_s(currency=self.currency, country=self.country)
        if try_valid:
            bag.try_valid_s()
        bag = bag.reload(
            eager=("lines.product.images", "lines.product.brand"), map=True
        )
        return bag

    @appier.route("/api/bags/<str:key>/merge/<str:target>", "PUT", json=True)
    def merge(self, key, target):
        increment = self.field("increment", False, cast=bool)
        bag = budy.Bag.get(key=key)
        target = budy.Bag.get(key=target)
        bag.merge_s(target.id, increment=increment)
        bag = bag.reload(map=True)
        return bag

    @appier.route("/api/bags/<str:key>/lines", "POST", json=True)
    def add_line(self, key):
        line = budy.BagLine.new()
        line.ensure_size_s()
        line.save()
        bag = budy.Bag.get(key=key)
        bag.lines.append(line)
        bag.save()
        line = line.reload(map=True)
        return line

    @appier.route("/api/bags/<str:key>/lines/<int:line_id>", "DELETE", json=True)
    def remove_line(self, key, line_id):
        bag = budy.Bag.get(key=key)
        bag.remove_line_s(line_id)
        bag = bag.reload(map=True)
        return bag

    @appier.route("/api/bags/<str:key>/lines/add_update", "POST", json=True)
    def add_update_line(self, key):
        increment = self.field("increment", True, cast=bool)
        line = budy.BagLine.new()
        line.ensure_size_s()
        bag = budy.Bag.get(key=key)
        line = bag.add_update_line_s(line, increment=increment)
        line = line.reload(map=True)
        return line

    @appier.route("/api/bags/<str:key>/empty", "GET", json=True)
    def empty(self, key):
        bag = budy.Bag.get(key=key)
        bag.empty_s()
        bag = bag.reload(map=True)
        return bag

    @appier.route("/api/bags/<str:key>/order", "GET", json=True)
    def order(self, key):
        bag = budy.Bag.get(key=key)
        order = bag.to_order_s()
        order = order.reload(map=True)
        return order



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Budy
# Copyright (c) 2008-2024 Hive Solutions Lda.
#
# This file is part of Hive Budy.
#
# Hive Budy is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Budy is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Budy. If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__copyright__ = "Copyright (c) 2008-2024 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

import commons

import appier

from . import base
from . import currency as _currency


class Measurement(base.BudyBase):
    name = appier.field(index=True, default=True)

    value = appier.field(type=int, index=True)

    value_s = appier.field(index=True)

    weight = appier.field(
        type=commons.Decimal,
        index=True,
        observations="""The weight of the current measurement in
        a unit defined by convention (defined before-hand)""",
    )

    quantity_hand = appier.field(type=commons.Decimal, index=True)

    quantity_reserved = appier.field(type=commons.Decimal, index=True)

    price = appier.field(
        type=commons.Decimal,
        index=True,
        initial=commons.Decimal(0.0),
        observations="""Main retail price to be used for
        a possible sale transaction of the measurement (includes taxes)""",
    )

    price_compare = appier.field(
        type=commons.Decimal,
        index=True,
        initial=commons.Decimal(0.0),
        observations="""The price that is going to be used
        as the base for discount calculation purposes""",
    )

    taxes = appier.field(type=commons.Decimal, index=True, initial=commons.Decimal(0.0))

    currency = appier.field(index=True)

    product = appier.field(type=appier.reference("Product", name="id"))

    @classmethod
    def validate(cls):
        return super(Measurement, cls).validate() + [
            appier.not_null("name"),
            appier.not_empty("name"),
            appier.not_null("value"),
            appier.gte("price", 0.0),
            appier.gte("taxes", 0.0),
            appier.not_null("product"),
        ]

    @classmethod
    def list_names(cls):
        return ["id", "name", "value"]

    @classmethod
    def order_name(cls):
        return ["id", -1]

    @classmethod
    def from_omni(
        cls,
        merchandise,
        sub_product=None,
        inventory_line=None,
        inventory_lines=None,
        name="size",
        currency="EUR",
        strip=True,
        path=True,
        force=False,
    ):
        from . import product

        # tries to retrieve the various elements that are going to be used
        # for the proper construction of the measurement from an omni
        # sub product entity (as expected)
        sub_product = sub_product or merchandise
        parent = sub_product["product"]
        object_id = sub_product["object_id"]
        weight = sub_product["weight"]
        modify_date = merchandise["modify_date"]
        company_product_code = merchandise["company_product_code"]
        metadata = merchandise["metadata"] or dict()
        price_compare = metadata.get("compare_price") or None
        discount = metadata.get("discount") or None

        # verifies if an inventory line has been provided, if that's the case
        # it's possible to determine a proper modification date for the sub product
        # taking into account also the modification date of its inventory line
        if inventory_line:
            modify_date_line = inventory_line["modify_date"]
            if modify_date_line > modify_date:
                modify_date = modify_date_line

        # tries to retrieve the parent product for this measurement using the
        # associated company product code as reference if there's no such parent
        # product then it's not possible to continue with the import operation
        _product = product.Product.get(
            product_id=parent["company_product_code"], raise_e=False
        )
        if not _product:
            return None

        # in case the discount at a merchandise level is not defined
        # then tries to use the one coming from the (parent) product
        if discount == None:
            discount = _product.meta.get("discount", None)

        # creates the stocks list in case there are valid inventory lines being
        # passed on the current measurement update/creation
        stocks = None if inventory_lines == None else []

        # iterates over the complete set of available inventory lines to build the
        # associated stock dictionary with the information on the stock point, this
        # is going to be added to the list of stocks to the measurement
        for inventory_line in inventory_lines if inventory_lines else []:
            stock_on_hand = inventory_line.get("stock_on_hand", 0)
            stock_reserved = inventory_line.get("stock_reserved", 0)
            stock_in_transit = inventory_line.get("stock_in_transit", 0)
            retail_price = inventory_line.get("retail_price", {}).get("value", 0.0)
            functional_unit = inventory_line.get("functional_unit", None)

            is_valid = functional_unit and functional_unit.get("status") == 1
            if not is_valid:
                continue

            stock_m = dict(
                store_id=functional_unit["object_id"],
                store_name=functional_unit["name"],
                stock_on_hand=stock_on_hand,
                stock_reserved=stock_reserved,
                stock_in_transit=stock_in_transit,
                retail_price=retail_price,
            )
            stocks.append(stock_m)

        # splits the provided company product code into its base part
        # and the sub code part (to be used as the measure value)
        value = company_product_code.split("-", 1)[1]

        # sets the string based value of the measurement as the raw
        # value of company product code split (as expected) note that
        # the strip flag controls if extra zeroes to the left should
        # be removed from the value (avoids extra values)
        value_s = value.lstrip("0") if strip else value

        # tries converts the value into an integer value, falling back
        # to the absolute hash value of it in case there's an error
        try:
            value = int(value)
        except ValueError:
            value = cls._hash(value)

        # tries to retrieve a measurement that is considered to be equivalent
        # to the one described by the associated subproduct in case it does
        # not exists creates a new instance that is going to be populate
        measurement = cls.get(
            product=_product.id, name=name, value=value, raise_e=False
        )
        if not measurement:
            measurement = cls()

        measurement.name = name
        measurement.value = value
        measurement.value_s = value_s
        measurement.weight = weight
        measurement.price_compare = price_compare
        measurement.currency = currency
        measurement.product = _product

        meta = dict(
            object_id=object_id,
            company_product_code=company_product_code,
            modify_date=modify_date,
            discount=discount,
        )
        if hasattr(measurement, "meta") and measurement.meta:
            measurement.meta.update(meta)
        else:
            measurement.meta = meta
        if not stocks == None:
            measurement.meta["stocks"] = stocks

        if "stock_on_hand" in merchandise or force:
            measurement.quantity_hand = merchandise.get("stock_on_hand", 0.0)
        if "retail_price" in merchandise or force:
            # "grabs" the retail price from the original merchandise entity
            # from Omni to be used as the base calculus
            retail_price = merchandise.get("retail_price", 0.0)

            # stores the "original" retail price in the measurement's metadata
            # storage may be needed latter for update operations
            measurement.meta["retail_price"] = retail_price

        if "price" in merchandise or force:
            # "grabs" the (untaxed) price from the original merchandise entity
            # from Omni to be used as the base calculus
            untaxed_price = merchandise.get("price", 0.0)

            # stores the "original" untaxed price in the measurement's metadata
            # storage may be needed latter for update operations
            measurement.meta["untaxed_price"] = untaxed_price

        # in case all of the required "original" financial information (prices)
        # is available then the price, taxes and price compare are calculated
        if "retail_price" in measurement.meta and "untaxed_price" in measurement.meta:
            untaxed_price = (
                _currency.Currency.round(
                    measurement.meta["untaxed_price"] * ((100.0 - discount) / 100.0),
                    currency,
                )
                if discount
                else measurement.meta["untaxed_price"]
            )
            measurement.price = (
                _currency.Currency.round(
                    measurement.meta["retail_price"] * ((100.0 - discount) / 100.0),
                    currency,
                )
                if discount
                else measurement.meta["retail_price"]
            )
            measurement.taxes = measurement.price - untaxed_price
            if not measurement.price_compare and discount:
                measurement.price_compare = measurement.meta["retail_price"]

        # returns the "final" measurement instance to the caller so that it's possible
        # to properly save the newly generated measurement instance according to omni
        return measurement

    @classmethod
    def _hash(cls, value, max_size=8):
        counter = 0
        for index in range(len(value)):
            value_i = appier.legacy.ord(value[index])
            counter += value_i * pow(256, index)
        if not max_size:
            return counter
        modulus = pow(256, max_size)
        counter = counter % modulus
        return counter

    def pre_delete(self):
        base.BudyBase.pre_delete(self)
        if not self.product:
            return
        if not hasattr(self.product, "measurements"):
            return
        if not self in self.product.measurements:
            return
        self.product.measurements.remove(self)
        self.product.save()

    def get_price(self, currency=None, country=None, attributes=None):
        return self.price

    def get_taxes(self, currency=None, country=None, attributes=None):
        return self.taxes

    def get_currency(self, currency=None):
        return currency

    def get_size(self, currency=None, country=None, attributes=None):
        return None, None

    @property
    def product_id(self):
        if self._product_id_meta:
            return self._product_id_meta
        return None

    @property
    def short_description(self):
        return self.product.short_description

    @property
    def quantity(self):
        return self.quantity_hand

    @property
    def discount(self):
        if not self.price:
            return commons.Decimal(0.0)
        if not self.price_compare:
            return commons.Decimal(0.0)
        return self.price_compare - self.price

    @property
    def discount_percent(self):
        if not self.discount:
            return commons.Decimal(0.0)
        return self.discount / self.price_compare * commons.Decimal(100.0)

    @property
    def is_discounted(self):
        return self.discount > 0.0

    @property
    def is_discountable(self):
        return self.product.is_discountable

    @property
    def is_price_provided(self):
        return False

    @appier.operation(name="Fix")
    def fix_s(self):
        if not self.exists():
            return
        self._fix_value_s()

    @appier.operation(name="Duplicate", factory=True)
    def duplicate_s(self):
        cls = self.__class__
        measurement = cls(
            product=self.product.id,
            name=self.name,
            value=self.value,
            value_s=self.value_s,
            quantity_hand=self.quantity_hand,
            price=self.price,
            currency=self.currency,
            meta=self.meta,
        )
        measurement.save()
        self.product.measurements.append(measurement)
        self.product.save()
        return measurement

    @property
    def _product_id_meta(self):
        if not self.meta:
            return None
        return self.meta.get("company_product_code", None)

    def _fix_value_s(self):
        cls = self.__class__
        try:
            self.value = int(self.value)
        except ValueError:
            self.value = self._hash(self.value)
        self.save()

    def _fix_invalid_s(self):
        is_valid = hasattr(self, "parent") and not self.parent == None
        if is_valid:
            return
        self.delete()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Budy
# Copyright (c) 2008-2024 Hive Solutions Lda.
#
# This file is part of Hive Budy.
#
# Hive Budy is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Budy is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Budy. If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__copyright__ = "Copyright (c) 2008-2024 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

import appier

from . import base


class Group(base.BudyBase):
    name = appier.field(
        index=True,
        default=True,
        observations="""The primary identifier of the group, can
        be used as the textual representation of it""",
    )

    title = appier.field(
        index=True,
        observations="""Text value to be used in the representation
        of the group, replaces the name visually""",
    )

    order = appier.field(type=int, index=True)

    labels = appier.field(type=list)

    image_url = appier.field(index=True, meta="image_url", description="Image URL")

    new_in = appier.field(type=bool)

    exclusive = appier.field(type=bool)

    images = appier.field(type=appier.references("Media", name="id"))

    @classmethod
    def validate(cls):
        return super(Group, cls).validate() + [
            appier.not_null("name"),
            appier.not_empty("name"),
        ]

    @classmethod
    def list_names(cls):
        return ["id", "name", "enabled"]

    @classmethod
    def is_abstract(cls):
        return True

    @classmethod
    def ensure_s(cls, name):
        group = cls.get(name=name, raise_e=False)
        if group:
            return group
        group = cls(name=name)
        group.save()
        return group

    def pre_validate(self):
        base.BudyBase.pre_validate(self)
        self.build_images()

    def pre_save(self):
        base.BudyBase.pre_validate(self)
        self.update_label(self.new_in, "new_in")
        self.update_label(self.exclusive, "exclusive")

    def build_images(self):
        thumbnail = self.get_image(size="thumbnail", order=1)
        thumbnail = thumbnail or self.get_image(size="thumbnail")
        image = self.get_image(size="large", order=1)
        image = image or self.get_image(size="large")
        self.thumbnail_url = thumbnail.get_url() if thumbnail else None
        self.image_url = image.get_url() if image else None

    def get_image(self, size=None, order=None):
        for image in self.images:
            is_size = size == None or image.size == size
            if not is_size:
                continue
            is_order = order == None or image.order == order
            if not is_order:
                continue
            return image
        return None

    def update_label(self, value, name):
        if value:
            if name in self.labels:
                return
            self.labels.append(name)
        else:
            if not name in self.labels:
                return
            self.labels.remove(name)

    @appier.operation(
        name="Add Image",
        parameters=(("Image", "image", appier.reference("Media", name="id")),),
    )
    def add_image_s(self, image):
        if not image:
            return
        if image in self.images:
            return
        self.images.append(image)
        self.save()

    @appier.operation(
        name="Remove Image",
        parameters=(("Image", "image", appier.reference("Media", name="id")),),
    )
    def remove_image_s(self, image):
        if not image:
            return
        if not image in self.images:
            return
        self.images.remove(image)
        self.save()

    @appier.operation(
        name="Upload Image",
        parameters=(
            ("File", "file", "file"),
            ("Label", "label", str, "main"),
            ("Size", "size", str, "large"),
        ),
    )
    def upload_image_s(self, file, label, size):
        from . import media

        if not file:
            return
        media = media.Media(
            description=self.name,
            label=label,
            order=1,
            size=size,
            file=appier.File(file),
        )
        media.save()
        self.images.append(media)
        self.save()

    @appier.view(name="Products")
    def orders_v(self, *args, **kwargs):
        from . import product

        cls = self.__class__
        plural = cls._underscore(plural=True)
        kwargs["sort"] = kwargs.get("sort", [("created", -1)])
        kwargs.update({plural: {"$all": [self.id]}})
        return appier.lazy_dict(
            model=product.Product,
            kwargs=kwargs,
            entities=appier.lazy(lambda: product.Product.find(*args, **kwargs)),
            page=appier.lazy(lambda: product.Product.paginate(*args, **kwargs)),
        )



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Budy
# Copyright (c) 2008-2024 Hive Solutions Lda.
#
# This file is part of Hive Budy.
#
# Hive Budy is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Budy is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Budy. If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__copyright__ = "Copyright (c) 2008-2024 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

import appier

from . import group


class Section(group.Group):
    """
    The section entity that represents an aggregation of groups
    that together define a proper unit of e-commerce (eg: watches
    vs jewelry on an e-commerce that sells both).

    Can be used for the creation of filtered "views" of the products
    so that partially disjoint groups are created.
    """

    context_fields = appier.field(
        type=list,
        observations="""The name of the fields that are
        considered to define the context of the section and
        that as such are going to be used in the filtering
        process of the associated products""",
    )
    """ Field that can be used to control the context filtering
    for the section by listing the names of the fields that are
    going to be used in the query, in case no value is defined
    all of the non empty groups are going to be used in the "view" """

    invisible_fields = appier.field(
        type=list,
        observations="""The name of the fields (groups) that
        although being used as part of the context selection for
        the section should not be displayed in the "normal" listing
        UI for the section""",
    )
    """ Field that controls the groups that are going to be hidden
    from the UI although being used for the context filtering"""

    genders = appier.field(type=list)

    collections = appier.field(type=appier.references("Collection", name="id"))

    categories = appier.field(type=appier.references("Category", name="id"))

    colors = appier.field(type=appier.references("Color", name="id"))

    brands = appier.field(type=appier.references("Brand", name="id"))

    seasons = appier.field(type=appier.references("Season", name="id"))

    @classmethod
    def validate(cls):
        return super(Section, cls).validate() + [
            appier.not_null("name"),
            appier.not_empty("name"),
        ]

    @classmethod
    def list_names(cls):
        return ["id", "name"]

    @appier.operation(
        name="Add Collection",
        parameters=(
            ("Collection", "collection", appier.reference("Collection", name="id")),
        ),
    )
    def add_collection_s(self, collection):
        if not collection:
            return
        self.collections.append(collection)
        self.save()

    @appier.operation(
        name="Remove Collection",
        parameters=(
            ("Collection", "collection", appier.reference("Collection", name="id")),
        ),
    )
    def remove_collection_s(self, collection):
        if not collection:
            return
        self.collections.remove(collection)
        self.save()

    @appier.operation(name="Clear Groups", level=2)
    def clear_groups_s(self):
        self.collections = []
        self.categories = []
        self.colors = []
        self.brands = []
        self.seasons = []
        self.save()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Budy
# Copyright (c) 2008-2024 Hive Solutions Lda.
#
# This file is part of Hive Budy.
#
# Hive Budy is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Budy is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Budy. If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__copyright__ = "Copyright (c) 2008-2024 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

import appier

from . import base


class Store(base.BudyBase):
    """
    Represents a physical store entity that can be used
    for the operations like instore pick-up, affecting
    the checkout process.
    """

    name = appier.field(index=True, default=True)

    address = appier.field(type=appier.reference("Address", name="id"), eager=True)

    currency_code = appier.field(index=True)

    checkout_mode = appier.field(index=True)

    restrict_mode = appier.field(type=bool, index=True, initial=True)

    @classmethod
    def validate(cls):
        return super(Store, cls).validate() + [
            appier.not_null("name"),
            appier.not_empty("name"),
            appier.string_eq("currency_code", 3),
        ]

    @classmethod
    def list_names(cls):
        return ["id", "name", "currency_code", "checkout_mode"]

    @property
    def is_restricted(self):
        return self.restrict_mode



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Budy
# Copyright (c) 2008-2024 Hive Solutions Lda.
#
# This file is part of Hive Budy.
#
# Hive Budy is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Budy is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Budy. If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__copyright__ = "Copyright (c) 2008-2024 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

import appier

import budy


class BaseController(appier.Controller):
    @appier.route("/", "GET")
    def index(self):
        return self.redirect(self.url_for("admin.index"))

    @appier.route("/index_store", "GET")
    def index_store(self):
        return self.redirect(self.url_for("order.me"))

    @appier.route("/signin", "GET")
    def signin(self):
        next = self.field("next")
        error = self.field("error")
        return self.template("signin.html.tpl", next=next, error=error)

    @appier.route("/signin", "POST")
    def login(self):
        username = self.field("username")
        password = self.field("password")
        next = self.field("next")
        try:
            account = budy.BudyAccount.login(username, password)
        except appier.AppierException as error:
            return self.template(
                "signin.html.tpl", next=next, username=username, error=error.message
            )

        account._set_account()

        return self.redirect(next or self.url_for(self.login_redirect))

    @appier.route("/signout", "GET")
    def signout(self):
        next = self.field("next")
        budy.BudyAccount._unset_account()
        return self.redirect(next or self.url_for(self.logout_redirect))
